@@ -38,6 +38,13 @@ impl Yozuk {
         Default::default()
     }
 
+    pub fn parse_tokens(text: &str) -> Vec<Token> {
+        let tokens = shell_words::split(text)
+            .ok()
+            .unwrap_or_else(|| text.split_whitespace().map(str::to_string).collect());
+        tokens.into_iter().map(|token| tk!(token)).collect()
+    }
+
     pub fn get_commands(
         &self,
         tokens: &[Token],
@@ -119,9 +126,10 @@ impl Yozuk {
 
         let mut errors = Vec::new();
         for (args, command) in commands {
+            let name = args.args[0].clone();
             match command.run(args, streams) {
                 Ok(result) => return Ok(result),
-                Err(err) => errors.push(err),
+                Err(err) => errors.push(err.into_output(name)),
             }
         }
 
@@ -262,3 +270,25 @@ struct CommandCache {
     translators: Vec<Box<dyn Translator>>,
     command: Box<dyn Command>,
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_parse_tokens() {
+        assert_eq!(
+            Yozuk::parse_tokens(" What's   the time "),
+            tk!(["What's", "the", "time"])
+        );
+        assert_eq!(
+            Yozuk::parse_tokens(r#" "Hello world" to md5 "#),
+            tk!(["Hello world", "to", "md5"])
+        );
+        assert_eq!(
+            Yozuk::parse_tokens(r#" (1 + 1) * 2 "#),
+            tk!(["(1", "+", "1)", "*", "2"])
+        );
+        assert_eq!(Yozuk::parse_tokens(r#" " \" \" " "#), tk!([" \" \" "]));
+    }
+}
